# -*- coding: utf-8 -*-
'''
Created on 16.06.2012

@author: Jan Brohl
@license: Simplified BSD

Copyright (c) 2012, Jan Brohl
All rights reserved.

Redistribution and use in source and binary forms,
with or without modification,
are permitted provided that the following conditions are met:

    Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.

    Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
'''
from urllib2 import urlopen, Request
import re
import datetime
import codecs
datere = re.compile(u"([0-9]+)-([0-9]+)-([0-9]+)$")
timere = re.compile(u"([0-9]+):([0-9]+):([0-9]+)$")
datetimere = re.compile(u"([0-9]+)-([0-9]+)-([0-9]+).([0-9]+):([0-9]+):([0-9]+)$")
intre = re.compile(u"[+-]?0|(?:[1-9][0-9]*)$")
floatre = re.compile(u"((?:[+-]?0|(?:[1-9][0-9]*))?)[.,]([0-9]+)(?:[Ee]|(?:[*]10[^])([+-]?0|(?:[1-9][0-9]*)))?$") #TODO: mehr formate
numstartre = re.compile(u"[+-]?[.]?[0-9]")

accept = dict()
try:
    accept["deflate"] = codecs.getdecoder("zlib")
except LookupError:
    pass
try:
    accept["bz2"] = codecs.getdecoder("bz2")
except LookupError:
    pass


def geturl(url):
    """
    Read Unicode from URL
    """
    f = urlopen(Request(url, headers={"Accept-Encoding":"; ".join(accept.keys())}))
    try:
        s = f.read()
        i = f.info()
        if i.get("Content-Encoding") in accept:
            s = accept[i.get("Content-Encoding")](s)[0]
        return s.decode(i.getparam("charset"))
    finally:
        f.close()


def convert(s):
    """
    Convert String value if possible in a safe way
    """
    if isinstance(s, basestring) and numstartre.match(s) is not None:
        m = datetimere.match(s)
        if m is not None:
            return datetime.datetime(*(int(g) for g in m.groups()))
        m = datere.match(s)
        if m is not None:
            return datetime.date(*(int(g) for g in m.groups()))
        m = timere.match(s)
        if m is not None:
            return datetime.time(*(int(g) for g in m.groups()))
        m = intre.match(s)
        if m is not None:
            return int(m.group())
        m = floatre.match(s)
        if m is not None:
            if len(m.groups()) == 3:
                return float("%s.%se%s" % tuple((g or "0") for g in m.groups()))
            return float("%s.%s" % tuple((g or "0") for g in m.groups()))
    return s


def as_list(mapping):
    """
    Convert dict to list if possible (if mapping.keys()==range(len(mapping)))
    else return dict
    """
    try:
        return list(mapping[i] for i in range(len(mapping)))
    except (KeyError, IndexError):
        return mapping
